# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_API_Integration.ipynb.

# %% auto 0
__all__ = ['model_engine', 'Role', 'create_message', 'generate_message', 'Speaker']

# %% ../nbs/02_API_Integration.ipynb 2
import datetime
import os
import json
import openai
from fastcore.basics import store_attr
from fastcore.test import test_eq
from fastcore.script import call_parse

# %% ../nbs/02_API_Integration.ipynb 3
openai.api_key = os.environ['OPENAI_API_KEY']
model_engine = "gpt-3.5-turbo"

# %% ../nbs/02_API_Integration.ipynb 4
from enum import Enum

# %% ../nbs/02_API_Integration.ipynb 5
class Role(Enum):
    "This corresponds to the roles that openai allows for the ChatGPT API"
    SYSTEM = "system"
    USER = "user"
    ASSISTANT = "assistant"
    FUNCTION = "function"

# %% ../nbs/02_API_Integration.ipynb 7
def create_message(role_type:Role, # Whether this message is the system, user, or assistant talking
                   content:str # A string that can be used 
                  ):
    return {"role":role_type.value, "content": content}

# %% ../nbs/02_API_Integration.ipynb 20
@call_parse
def generate_message(system_content:str=None, # A system message that will be given to the API
                     user_content:str=None, # A user message that will be give to the API
                     model_engine="gpt-3.5-turbo", # The model from openai that will be used
                     messages:list=None # a list of messages to optionally keep track of state
                    ) -> json: # The message returned by openai will be returned as a json
    "Start a conversation with chatgpt!"
    if messages is None: messages = []
    if system_content is not None: messages.append(create_message(Role.SYSTEM, system_content))
    if user_content is not None: messages.append(create_message(Role.USER, user_content))
    if messages == []:
        return {"error":"No message was sent to openai because messages was empty.  Pass `system_content`, `user_content`, or a list of `messages` to start the conversations"}
    completion = openai.ChatCompletion.create(
        model=model_engine,
        messages=messages,
    )
    return completion.to_dict_recursive()

# %% ../nbs/02_API_Integration.ipynb 23
class Speaker:
    "A speaker is somebody that will talk about things!"
    def __init__(self, name:str, backstory:str=None, mannerisms:str=None, relationships:dict=None, model_engine="gpt-3.5-turbo"): # Test
        store_attr()
        self.messages=[]
        self.setup_backstory()
        self.setup_mannerisms()
        self.setup_universe()

    def setup_universe(self):
        # self.messages.append(create_message(Role.SYSTEM, f'you are one of a handful of members in an improv group in front of a live audience.'))
        self.messages.append(create_message(Role.SYSTEM, f'Your message should be no more than a paragraph'))
        self.messages.append(create_message(Role.SYSTEM, f'Responses should like like this YOUR_NAME: Response to the previous messages.'))
        self.messages.append(create_message(Role.SYSTEM, f'If needed you can do actions by putting them in asterisks'))
        self.messages.append(create_message(Role.SYSTEM, f'Use markdown to make the output look pretty'))
        self.messages.append(create_message(Role.SYSTEM, f'You can only write your own viewpoint of the story. Never write the other persons response'))
    
    def setup_backstory(self):
        if self.backstory is None:
            self.messages.append(create_message(Role.SYSTEM, f'Your name is {self.name}. Choose a random backstory and make sure to tell me the backstory at the top of the next message'))
        else:
            self.messages.append(create_message(Role.SYSTEM, f'Your name is {self.name} and here is your backstory: {self.backstory}'))

    def setup_mannerisms(self):
        if self.mannerisms is None:
            self.messages.append(create_message(Role.SYSTEM, f'Choose some random mannerisms and make sure to tell me what the mannerisms are at the top of the next message'))
        else:
            self.messages.append(create_message(Role.SYSTEM, f'Here are some of your mannerisms: {self.mannerisms}'))

    def setup_scene(self, scene:str):
        self.messages.append(create_message(Role.SYSTEM, scene))
        return "**SCENE PLOT: "+scene+"**"
    
    def listen_to_input(self,inp):
        self.messages.append(create_message(Role.USER, inp))

    def talk(self, max_tokens:int=300):
        completion = openai.ChatCompletion.create(
            model=self.model_engine,
            messages=self.messages,
            max_tokens=max_tokens
        )
        self.messages.append(create_message(Role.ASSISTANT, completion.choices[0].message.content))
        return completion.choices[0].message.content
